package de.ozgcloud.nachrichten.postfach.osiv2;

import static de.ozgcloud.nachrichten.postfach.osiv2.factory.JwtFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockserver.mock.OpenAPIExpectation.*;
import static org.mockserver.model.HttpRequest.*;

import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.RegisterExtension;
import org.mockserver.client.MockServerClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.DynamicPropertyRegistry;
import org.springframework.test.context.DynamicPropertySource;
import org.springframework.test.context.TestPropertySource;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.extension.Jwt;
import de.ozgcloud.nachrichten.postfach.osiv2.extension.OsiMockServerExtension;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachNachrichtTestFactory;
import lombok.SneakyThrows;

@SpringBootTest(classes = TestApplication.class, webEnvironment = SpringBootTest.WebEnvironment.NONE)
@ActiveProfiles("itcase")
@TestPropertySource(properties = {
		"ozgcloud.osiv2-postfach.http-proxy.enabled=false",
})
public class OsiPostfachRemoteServiceITCase {

	@RegisterExtension
	static final OsiMockServerExtension OSI_MOCK_SERVER_EXTENSION = new OsiMockServerExtension();

	private final PostfachNachricht postfachNachricht = PostfachNachrichtTestFactory.create();

	@Autowired
	private OsiPostfachRemoteService osiPostfachRemoteService;

	@DynamicPropertySource
	static void dynamicProperties(DynamicPropertyRegistry registry) {
		registry.add("spring.security.oauth2.client.provider.osi2.token-uri", OSI_MOCK_SERVER_EXTENSION::getAccessTokenUrl);
		registry.add("spring.security.oauth2.client.registration.osi2.scope", () -> CLIENT_SCOPES);
		registry.add("spring.security.oauth2.client.registration.osi2.client-id", () -> CLIENT_ID);
		registry.add("ozgcloud.osiv2-postfach.api.url", OSI_MOCK_SERVER_EXTENSION::getPostfachFacadeUrl);
		registry.add("ozgcloud.osiv2-postfach.api.resource", () -> RESOURCE_URN);
	}

	private MockServerClient postfachFacadeMockClient;

	@BeforeEach
	@SneakyThrows
	public void setup() {
		postfachFacadeMockClient = OSI_MOCK_SERVER_EXTENSION.getPostfachFacadeMockClient();
	}

	@DisplayName("send message")
	@Nested
	class TestSendMessage {

		@DisplayName("should send dummy request with jwt")
		@Test
		@SneakyThrows
		void shouldSendDummyRequestWithJwt() {
			postfachFacadeMockClient.upsert(
					openAPIExpectation()
							.withSpecUrlOrPayload(Files.readString(Path.of("spec", "postfach-api-facade.yaml")))
							.withOperationsAndResponses(Map.of(
									"SendMessage", "200"
							))
			);

			osiPostfachRemoteService.sendMessage(postfachNachricht);

			var requests = postfachFacadeMockClient.retrieveRecordedRequests(request());
			assertThat(requests).hasSize(1);
			var jwt = Jwt.parseAuthHeaderValue(
					requests[0].getHeader("Authorization").getFirst()
			);
			assertThat(jwt.body().read("$.client_id", String.class)).isEqualTo(CLIENT_ID);
			assertThat(jwt.body().read("$.aud", String.class)).isEqualTo(RESOURCE_URN);
		}

	}
}
