package de.ozgcloud.nachrichten.postfach.osiv2.extension;

import static de.ozgcloud.nachrichten.postfach.osiv2.factory.JwtFactory.*;
import static org.mockserver.matchers.Times.*;
import static org.mockserver.model.Header.*;
import static org.mockserver.model.HttpRequest.*;
import static org.mockserver.model.HttpResponse.*;
import static org.mockserver.model.Parameter.*;
import static org.mockserver.model.ParameterBody.*;

import org.junit.jupiter.api.extension.AfterAllCallback;
import org.junit.jupiter.api.extension.AfterEachCallback;
import org.junit.jupiter.api.extension.BeforeAllCallback;
import org.junit.jupiter.api.extension.BeforeEachCallback;
import org.junit.jupiter.api.extension.ExtensionContext;
import org.mockserver.client.MockServerClient;
import org.testcontainers.containers.MockServerContainer;
import org.testcontainers.containers.output.OutputFrame;
import org.testcontainers.utility.DockerImageName;

import de.ozgcloud.nachrichten.postfach.osiv2.factory.JwtFactory;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Getter
@RequiredArgsConstructor
public class OsiMockServerExtension implements BeforeAllCallback, AfterAllCallback, BeforeEachCallback, AfterEachCallback {

	private MockServerClient serviceKontoMockClient;
	private MockServerClient postfachFacadeMockClient;

	private static final DockerImageName MOCK_SERVER_IMAGE = DockerImageName.parse("mockserver/mockserver")
			.withTag("mockserver-5.15.0");

	private final MockServerContainer serviceKontoMockContainer = new MockServerContainer(MOCK_SERVER_IMAGE)
			.withLogConsumer(this::logContainerOutput);
	private final MockServerContainer postfachFacadeMockContainer = new MockServerContainer(MOCK_SERVER_IMAGE)
			.withLogConsumer(this::logContainerOutput);

	private void logContainerOutput(OutputFrame outputFrame) {
		var line = outputFrame.getUtf8String().stripTrailing();
		if (outputFrame.getType() == OutputFrame.OutputType.STDERR) {
			System.err.println(line);
		} else {
			System.out.println(line);
		}
	}

	@Override
	public void beforeAll(ExtensionContext context) {
		setupPostfachFacadeMock();
		setupServiceKontoMock();
	}

	@Override
	public void afterEach(ExtensionContext context) {
		postfachFacadeMockClient.reset();
		serviceKontoMockClient.reset();
	}

	@Override
	public void afterAll(ExtensionContext context) {
		serviceKontoMockClient.stop();
		serviceKontoMockContainer.stop();

		postfachFacadeMockClient.stop();
		postfachFacadeMockContainer.stop();
	}

	private void setupPostfachFacadeMock() {
		postfachFacadeMockContainer.start();
		postfachFacadeMockClient = new MockServerClient(
				postfachFacadeMockContainer.getHost(),
				postfachFacadeMockContainer.getServerPort()
		);
	}

	private void setupServiceKontoMock() {
		serviceKontoMockContainer.start();
		serviceKontoMockClient = new MockServerClient(
				serviceKontoMockContainer.getHost(),
				serviceKontoMockContainer.getServerPort()
		);
	}

	public String getAccessTokenUrl() {
		return getMockServerUrl(serviceKontoMockClient) + "/access-token";
	}

	public String getPostfachFacadeUrl() {
		return getMockServerUrl(postfachFacadeMockClient);
	}

	private String getMockServerUrl(MockServerClient mockClient) {
		return "http://" + mockClient.remoteAddress().getHostName() + ":" + mockClient.remoteAddress().getPort();
	}

	@Override
	public void beforeEach(ExtensionContext context) {
		serviceKontoMockClient
				.when(
						request()
								.withMethod("POST")
								.withPath("/access-token")
								.withHeaders(
										header("Content-Type", "application/x-www-form-urlencoded;charset=UTF-8")
								)
								.withBody(
										params(
												param("grant_type", "client_credentials"),
												param("client_id", CLIENT_ID),
												param("client_secret", "changeme"),
												param("scope", String.join(" ", CLIENT_SCOPES)),
												param("resource", RESOURCE_URN)
										)
								),
						exactly(1)
				)
				.respond(
						response()
								.withStatusCode(200)
								.withHeader("Content-Type", "application/json")
								.withBody(
										JwtFactory.createTokenResponse(
												JwtFactory.createAccessTokenExampleWithExpireIn(900)
										)
								)
				);
	}
}
